#!/usr/bin/python
import os, sys, time, gc
from qt import *

######### Query Memory Usage Functions ###############################
_proc_status = '/proc/%d/status' % os.getpid()
_scale = {'kB': 1024.0, 'mB': 1024.0*1024.0,
          'KB': 1024.0, 'MB': 1024.0*1024.0}

def _VmB(VmKey):
    global _proc_status, _scale

    VmKey = "Vm"+VmKey+":"
     # get pseudo file  /proc/<pid>/status
    try:
        t = open(_proc_status)
        v = t.read()
        t.close()
    except:
        return 0.0  # non-Linux?
     # get VmKey line e.g. 'VmRSS:  9999  kB\n ...'
    i = v.index(VmKey)
    v = v[i:].split(None, 3)  # whitespace
    if len(v) < 3:
        return 0.0  # invalid format?
     # convert Vm value to bytes
    return float(v[1]) * _scale[v[2]]


mem_types = ('RSS', 'Stk', 'Data', 'Lib', 'Size')
def memoryUsage():
    return "Current Memory Usage: " + " ".join(["%s:%s" % (k, _VmB(k)) for k in mem_types])

max_mem_by_type = {}
for t in mem_types:
    max_mem_by_type[t] = _VmB(t)

def check_leak(s):
    for mem_type, value in max_mem_by_type.items():
        current_mem = _VmB(mem_type)
        if current_mem > value:
            report("\tAT %s: MAX '%s' MEMORY INCREASED FROM %d TO %d (+%d)" % \
                  (s, mem_type, value, current_mem, current_mem-value))
            sys.stdout.flush()
            max_mem_by_type[mem_type] = current_mem

############################

def report(s):
  print s
  sys.stdout.flush()

class Main(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)

    def keyPressEvent(self, event):
        NUM_FORMS = 10000
        k = event.key()
        if k in (QKeyEvent.Key_Enter, QKeyEvent.Key_Return, QKeyEvent.Key_C):
            while 1:
              report("*** Creating %d QDialog's" % NUM_FORMS)
              check_leak("Pre-Creating")
              for x in range(NUM_FORMS):
                  f = QDialog()  # leaks with parent or not 
                  f = None
              report("End Creating\nForcing GC")
              col   = gc.collect()
              uncol = len(gc.garbage)
              if col or uncol:
                report("GC Collected: %d Uncollected: %d" % (col, uncol))
              report("End GC")
              check_leak("Post-Creating")
              report(memoryUsage()+"\n")
              if k != QKeyEvent.Key_C:
                  break
              else:
                  report("Sleeping 2 seconds...")
                  time.sleep(2)

        elif k == QKeyEvent.Key_M:
            report(memoryUsage())


app = QApplication(sys.argv)

w = Main()
app.setMainWidget(w)
w.show()
report("Focus the Window and press Enter for object creation / destruction, 'M' for current memory usage, or 'C' for continous creation\n")
report(memoryUsage())
app.exec_loop()


